#include <client.h>

/* This file handles various player related functions.  This includes
 * both things that operate on the player item, cpl structure, or
 * various commands that the player issues.
 *
 *  This file does most of the handling of commands from the client to
 *  server (see commands.c for server->client)
 *
 *  does most of the work for sending messages to the server
 *   Again, most of these appear self explanatory.  Most send a bunch of
 *   commands like apply, examine, fire, run, etc.  This looks like it
 *   was done by Mark to remove the old keypress stupidity I used. 
 */

/* This translates the numeric direction id's into the actual direction
 * commands.  This lets us send the actual command (ie, 'north'), which
 * makes handling on the server side easier.
 */

char *directions[9] = {"stay", "north", "northeast", "east", "southeast",
		"south","southwest", "west", "northwest"};


/*
 *  Initialiazes player item, information is received from server
 */
void new_player (long tag, char *name, long weight, long face)
{
    cpl.ob->tag    = tag;
    cpl.ob->nrof   = 1;
    copy_name (cpl.ob->name, name);
    cpl.ob->weight = (float) weight / 1000;
    cpl.ob->face   = face;
}

void look_at(int x, int y)
{
    char buf[MAX_BUF];

    sprintf(buf,"lookat %d %d", x, y);
    cs_write_string(csocket.fd, buf, strlen(buf));
}

void client_send_apply (int tag)
{
    char buf[MAX_BUF];

    sprintf(buf,"apply %d", tag);
    cs_write_string(csocket.fd, buf, strlen(buf));
}

void client_send_examine (int tag)
{
    char buf[MAX_BUF];

    sprintf(buf,"examine %d", tag);
    cs_write_string(csocket.fd, buf, strlen(buf));

}

/* Requests nrof objects of tag get moved to loc. */
void client_send_move (int loc, int tag, int nrof)
{
    char buf[MAX_BUF];

    sprintf(buf,"move %d %d %d", loc, tag, nrof);
    cs_write_string(csocket.fd, buf, strlen(buf));
}



void move_player(int dir) {
    char buf[MAX_BUF];

    sprintf(buf,"command %s", directions[dir]);
    cs_write_string(csocket.fd, buf, strlen(buf));
}


/* Fire & Run code.  The server handles repeating of these actions, so
 * we only need to send a run or fire command for a particular direction
 * once - we use the drun and dfire to keep track if we need to send
 * the full command.
 */
static int drun=-1, dfire=-1;

/* Fires in a specified direction.  Note that direction 0 is a valid
 * case - the fire is centered on the player.
 */

void stop_fire()
{
    if (cpl.input_state != Playing) return;
    cs_write_string(csocket.fd, "command fire_stop", strlen("command fire_stop"));
    dfire=-1;
}



void fire_dir(int dir) {
    char buf[MAX_BUF];

    if (cpl.input_state != Playing) return;
    if (dir!=dfire) {
	sprintf(buf,"command fire %d", dir);
	cs_write_string(csocket.fd, buf, strlen(buf));
	dfire=dir;
    }
}

void stop_run()
{
    cs_write_string(csocket.fd, "command run_stop", strlen("command run_stop"));
    drun=-1;
}

void run_dir(int dir) {
    char buf[MAX_BUF];

    if (dir!=drun) {
	sprintf(buf,"command run %d", dir);
	cs_write_string(csocket.fd, buf, strlen(buf));
	drun=dir;
    }
}


void send_command(char *command) {
    char buf[MAX_BUF];

    sprintf(buf,"command %d %s", cpl.count,command);
    cs_write_string(csocket.fd, buf, strlen(buf));
    cpl.count=0;
}

/* Show a basic help message */
void show_help() {
    draw_info("Client Side Commands", NDI_NAVY);
    draw_info("  bind        - bind a command to key", NDI_BLACK);
    draw_info("  unbind      - unbind a command, show bindings", NDI_BLACK);
    draw_info("  showicon    - draw status icons in inventory window", NDI_BLACK);
    draw_info("  showweight  - show weight in inventory/look windows", NDI_BLACK);
    draw_info("  scroll      - toggle scroll/wrap mode in info window", NDI_BLACK);
    draw_info("  magicmap    - show last received magicmap", NDI_BLACK);
    draw_info("  savewinpos  - save window positions (split mode", NDI_BLACK);
    draw_info("  savedefaults- save various defaults into ~/.crossfire/defaults", NDI_BLACK);
    draw_info("  inv         - show clients inventory (debug)", NDI_BLACK);
    draw_info("  show        - determine what type of items to show in inventory", NDI_BLACK);
}

/* This is an extended command (ie, 'who, 'whatever, etc).  In general,
 * we just send the command to the server, but there are a few that
 * we care about (bind, unbind)
 */

void extended_command(char *command) {
    char *cp = command, *cpnext;

    if ((cpnext = strchr(cp, ' '))!=NULL)
	*cpnext++ = '\0';


    if (!strcmp(cp, "bind"))
	bind_key(cpnext);
    else if (!strcmp(cp,"unbind"))
	unbind_key(cpnext);
    else if (!strcmp(cp,"showicons"))
	set_show_icon (cpnext);
    else if (!strcmp(cp,"showweight"))
	set_show_weight (cpnext);
    else if (!strcmp(cp,"scroll"))
	set_scroll(cpnext);
    else if (!strcmp(cp,"magicmap")) {
	cpl.showmagic=1;
	draw_magic_map();
    }
    else if (!strcmp(cp,"savewinpos")) {
	save_winpos();
    }
    else if (!strcmp(cp,"savedefaults")) {
	save_defaults();
    }
    else if (!strcmp(cp,"help")) {
	show_help();
	cs_write_string(csocket.fd, "command help", strlen("command help"));
    }
    else if (!strcmp(cp,"show")) {
	command_show(cpnext);
    }
    else if (!strcmp(cp,"inv")) /* inventory command is sended to server
				   for debugging purposes */
	print_inventory (cpl.ob);
    else {
	char buf[MAX_BUF];

	/* reconstruct the string */
	if (cpnext) *(--cpnext) = ' ';


	sprintf(buf,"command %s", command);
	cs_write_string(csocket.fd, buf, strlen(buf));
    }
}
 
